@@ -274,6 +274,12 @@ class Subcommands(object):
                   'action': "store_true",
                   'help': "Do not delete public access block from buckets" +
                           "(this way postrunjson and metrics reports will not be public)"},
+                 {'flag': ["-t", "--subnets"],
+                  'nargs': '+',
+                  'help': "subnet IDs"},
+                 {'flag': ["-r", "--security-groups"],
+                  'nargs': '+',
+                  'help': "security groups"},
                  {'flag': ["-q", "--quiet"],
                   'action': "store_true",
                   'help': "minimize standard output from deployment"}],
@@ -286,6 +292,12 @@ class Subcommands(object):
                  {'flag': ["-g", "--usergroup"],
                   'default': '',
                   'help': "Tibanna usergroup for the AWS Lambda function"},
+                 {'flag': ["-t", "--subnets"],
+                  'nargs': '+',
+                  'help': "subnet IDs"},
+                 {'flag': ["-r", "--security-groups"],
+                  'nargs': '+',
+                  'help': "security groups"},
                  {'flag': ["-q", "--quiet"],
                   'action': "store_true",
                   'help': "minimize standard output from deployment"}],
@@ -364,15 +376,19 @@ class Subcommands(object):
                           "To use this option, turn off option -B."},
                  {'flag': ["-U", "--ubuntu-base-image"],
                   'help': "The ID of the Ubuntu 20.04 image to build from (e.g. 'ami-0885b1f6bd170450c' for us-east-1). " +
-                          "To use this option, turn on the option -B."}]
+                          "To use this option, turn on the option -B."},
+                 {'flag': ["-r", "--replicate"],
+                  'help': "Enable to replicate across all regions defined by AMI_PER_REGION"}
+                 ]
         }
 
 
-def deploy_core(name, suffix=None, usergroup='', quiet=False):
+def deploy_core(name, suffix=None, usergroup='', quiet=False, subnets=None, security_groups=None):
     """
     New method of deploying packaged lambdas (BETA)
     """
-    API().deploy_core(name=name, suffix=suffix, usergroup=usergroup, quiet=quiet)
+    API().deploy_core(name=name, suffix=suffix, usergroup=usergroup, subnets=subnets,
+                      security_groups=security_groups, quiet=quiet)
 
 
 def run_workflow(input_json, sfn=TIBANNA_DEFAULT_STEP_FUNCTION_NAME, jobid='', do_not_open_browser=False, sleep=3):
@@ -396,11 +412,12 @@ def setup_tibanna_env(buckets='', usergroup_tag='default', no_randomize=False,
 
 def deploy_unicorn(suffix=None, no_setup=False, buckets='',
                    no_setenv=False, usergroup='', do_not_delete_public_access_block=False,
-                   deploy_costupdater=False, quiet=False):
+                   deploy_costupdater=False, subnets=None, security_groups=None, quiet=False):
     """deploy tibanna unicorn to AWS cloud"""
     API().deploy_unicorn(suffix=suffix, no_setup=no_setup, buckets=buckets, no_setenv=no_setenv,
                          usergroup=usergroup, do_not_delete_public_access_block=do_not_delete_public_access_block,
-                         deploy_costupdater=deploy_costupdater, quiet=quiet)
+                         deploy_costupdater=deploy_costupdater, subnets=subnets, security_groups=security_groups,
+                         quiet=quiet)
 
 
 def add_user(user, usergroup):
@@ -505,10 +522,10 @@ def cleanup(usergroup, suffix='', purge_history=False, do_not_remove_iam_group=F
 
 
 def create_ami(make_public=False, build_from_scratch=False, source_image_to_copy_from=None, source_image_region=None,
-               ubuntu_base_image=None):
+               ubuntu_base_image=None, replicate=False):
     print(API().create_ami(make_public=make_public, build_from_scratch=build_from_scratch,
                            source_image_to_copy_from=source_image_to_copy_from, source_image_region=source_image_region,
-                           ubuntu_base_image=ubuntu_base_image))
+                           ubuntu_base_image=ubuntu_base_image, replicate=replicate))
 
 
 def main(Subcommands=Subcommands):
