@@ -86,8 +86,11 @@ class UnicornInput(SerializableObject):
         if args.app_name and args.app_name in B.app_name_function_map:
             pass  # use benchmarking
         else:
-            if not cfg.ebs_size or cfg.ebs_size < 10:  # unset (set to 0) or <10GB
+            if not cfg.ebs_size:
                 cfg.ebs_size = 10  # if not set by user or benchmark, just use 10GB as default
+            # This one below may require revisiting - At least in some cases AWS now allows <10GB EBS.
+            if isinstance(cfg.ebs_size, int) and cfg.ebs_size < 10:
+                cfg.ebs_size = 10  # if user-specified value is less than 10G, use 10GB as default
             if not cfg.EBS_optimized:  # either false or unset
                 cfg.EBS_optimized = False  # False by default so t2 instances can be used
             if cfg.mem and cfg.cpu:
@@ -166,6 +169,8 @@ class Args(SerializableObject):
                 self.wdl_directory_local = ''
             if not hasattr(self, 'wdl_directory_url'):
                 self.wdl_directory_url = ''
+            if not hasattr(self, 'workflow_engine'):
+                self.workflow_engine = 'cromwell'
             if not self.wdl_directory_local and not self.wdl_directory_url:
                 errmsg = "either %s or %s must be provided in args" % ('wdl_directory_url', 'wdl_directory_local')
                 raise MissingFieldInInputJsonException(errmsg)
@@ -267,14 +272,15 @@ class Config(SerializableObject):
 
     def fill_default(self):
         # use benchmark or not
-        if not hasattr(self, 'instance_type') and (not hasattr(self, 'cpu') or not hasattr(self, 'mem')):
-            self.use_benchmark = True
-        elif not hasattr(self, 'ebs_size'):
-            self.use_benchmark = True 
-        else:
-            self.use_benchmark = False
+        if not hasattr(self, 'use_benchmark'):
+            if not hasattr(self, 'instance_type') and (not hasattr(self, 'cpu') or not hasattr(self, 'mem')):
+                self.use_benchmark = True
+            elif not hasattr(self, 'ebs_size'):
+                self.use_benchmark = True 
+            else:
+                self.use_benchmark = False
         # fill in default
-        for field in ['instance_type', 'EBS_optimized', 'cpu', 'ebs_iops', 'password', 'key_name',
+        for field in ['instance_type', 'EBS_optimized', 'cpu', 'ebs_iops', 'ebs_throughput', 'password', 'key_name',
                       'spot_duration', 'availability_zone', 'security_group', 'subnet']:
             if not hasattr(self, field):
                 setattr(self, field, '')
@@ -305,7 +311,9 @@ class Config(SerializableObject):
             self.mem_as_is = False
         if not hasattr(self, 'ebs_size_as_is'):  # if false, add 5GB overhead
             self.ebs_size_as_is = False
-
+        if not hasattr(self, 'ami_id'): 
+            self.ami_id = AMI_ID
+        
     def fill_internal(self):
         # fill internally-used fields (users cannot specify these fields)
         # script url
@@ -314,8 +322,7 @@ class Config(SerializableObject):
         self.json_bucket = self.log_bucket
 
     def fill_language_options(self, language='cwl_v1', singularity=False):
-        """fill in ami_id and language fields (these are also internal)"""
-        self.ami_id = AMI_ID
+        """fill in language fields (these are also internal)"""
         if singularity:
             self.singularity = True
         self.language = language
@@ -341,6 +348,10 @@ class Execution(object):
         self.input_size_in_bytes = self.get_input_size_in_bytes()
         if self.cfg.use_benchmark:
             self.benchmark = self.get_benchmarking(self.input_size_in_bytes)
+            logger.debug('self.benchmark = ' + str(self.benchmark))
+        else:
+            logger.debug('self.cfg.use_benchmark = ' + str(self.cfg.use_benchmark))
+        logger.debug('self.cfg.as_dict() = ' + str(self.cfg.as_dict()))
         self.init_instance_type_list()
         self.update_config_instance_type()
         self.update_config_ebs_size()
@@ -479,7 +490,6 @@ class Execution(object):
                                                    'parameters': benchmark_parameters})
         except Exception as e:
             try:
-                res
                 raise Exception("Benchmarking not working. : {}".format(str(res)))
             except:
                 raise Exception("Benchmarking not working. : None. %s" % str(e))
@@ -571,6 +581,7 @@ class Execution(object):
                 'main_wdl': args.wdl_main_filename,
                 'other_wdl_files': ','.join(args.wdl_child_filenames),
                 'wdl_url': args.wdl_directory_url,
+                'workflow_engine': args.workflow_engine
             })
         elif args.language == 'snakemake':
             app.update({
@@ -720,6 +731,11 @@ class Execution(object):
                                                        'VolumeType': 'gp3'}}]})
         if self.cfg.ebs_iops:    # io1 type, specify iops
             largs["BlockDeviceMappings"][0]["Ebs"]['Iops'] = self.cfg.ebs_iops
+        if self.cfg.ebs_throughput and self.cfg.ebs_type == 'gp3':
+            if self.cfg.ebs_throughput < 125 or self.cfg.ebs_throughput > 1000:
+                message = "Invalid EBS throughput. Specify a value between 125 and 1000."
+                raise EC2LaunchException(message)
+            largs["BlockDeviceMappings"][0]["Ebs"]['Throughput'] = self.cfg.ebs_throughput
         if self.cfg.ebs_size >= 16000:
             message = "EBS size limit (16TB) exceeded: (attempted size: %s)" % self.cfg.ebs_size
             raise EC2LaunchException(message)
